import magic
from django import template
from django.core import serializers
from django.template.defaultfilters import safe
from ..models.validators.file import VALID_FILE_TYPES

register = template.Library()


@register.filter
def json_dump(values):
    # Return values as JSON data
    data = list(values)
    data = serializers.serialize('json', data)
    return safe(data)


@register.filter
def file_mime(file):
    # Get the mime type of a file (E.g. image/jpeg, text/plain)
    mime = magic.from_file(file, mime=True)
    return mime


@register.filter
def file_type(mime):
    # TODO: fix implementation of filter
    try:
        category, t = mime.split('/')

        if category == 'application':
            if t == 'vnd.apple.keynote':
                return 'powerpoint'
            if t in ['vnd.apple.pages', 'msword']:
                return 'word'
            if t in ['vnd.apple.numbers', 'csv']:
                return 'excel'
            if t in ['zip']:
                return 'folder'
            return t

        if category in ['image', 'audio', 'video', 'text']:
            if category == 'text':
                if t in ['plain', 'rtf', 'richtext']:
                    return 'alt'
                else:
                    return 'code'
            return category
    except ValueError:
        return 'code'


@register.filter
def file_size(file, unit):
    """
    Convert the size from bytes to other units like KB, MB or GB

    Adapted from:
    https://thispointer.com/python-get-file-size-in-kb-mb-or-gb-human-readable-format/
    """

    base = 1024

    if unit == 'KB':
        size = file.size/base
    elif unit == 'MB':
        size = file.size/(base**2)
    elif unit == 'GB':
        size = file.size/(base**3)
    else:
        size = file.size

    return f'{round(size, 2)} {unit}'


@register.filter
def total_file_sizes(files, unit):
    total = 0
    base = 1024

    for file in files:
        if unit == 'KB':
            size = file.file.size / base
        elif unit == 'MB':
            size = file.file.size / (base ** 2)
        elif unit == 'GB':
            size = file.file.size / (base ** 3)
        else:
            size = file.file.size
        total += size
    return f'{round(total, 2)} {unit}'


@register.filter
def file_name(file):
    return file.name.split('/')[-1]



from django.test import TestCase
from ..save import Save


class SaveTestCase(TestCase):
    def setUp(self):
        """
        Create objects here...

        Example:
        Save.objects.create()
        """

        pass

    def test_create_save(self):
        """
        Run assertions here...

        Example:
        save = Save.objects.create()
        self.assertEqual(save.id, 1)
        """

        pass



from rest_framework import serializers
from ..serializers import CommentSerializer
from ..models import File


class FileSerializer(serializers.ModelSerializer):

    comments = CommentSerializer(many=True)

    class Meta:
        model = File
        exclude = (
            'uuid',
            'object_id',
            'content_type',
        )



from django.db import models
from django.urls import reverse
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from .abstracts import ArvestustRecord


class Tag(ArvestustRecord):
    content = models.CharField(max_length=64, verbose_name=_('content'))

    class Meta:
        db_table = 'arvestust_tags'
        unique_together = ('object_id', 'content_type', 'content',)

    def clean(self):
        super(Tag, self).clean()
        if Tag.objects.filter(
                content=self.content,
                object_id=self.object_id,
        ).count() > 0:
            raise ValidationError(_('This is a duplicate entry'))

    def __str__(self):
        return f'{self.content}'

    def get_absolute_url(self):
        return reverse('arvestust:tag-detail', kwargs={'slug': self.slug})



from .record import ArvestustRecord
from ..managers.user_record import UserRecordManager


class UserRecord(ArvestustRecord):
    user = models.ForeignKey(
        get_user_model(),
        related_name='%(class)s',
        on_delete=models.CASCADE,
        verbose_name=_('user')
    )

    class Meta:
        abstract = True
        unique_together = ('object_id', 'user',)
        indexes = [models.Index(fields=['user', 'created_at', 'object_id'])]

    objects = UserRecordManager()



from django.test import TestCase
from ..models.file import File


class FileTestCase(TestCase):
    def setUp(self):
        """
        Create objects here...

        Example:
        File.objects.create()
        """
        pass

    def test_create_file(self):
        """
        Run assertions here...

        Example:
        file = File.objects.create()
        self.assertEqual(file.id, 1)
        """
        pass



# Generated by Django 2.2.13 on 2020-07-29 16:39

import arvestust.models.abstracts.file
import arvestust.models.validators.file
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('arvestust', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='file',
            field=models.FileField(storage='packager.storage.PublicFileStorage', upload_to=arvestust.models.abstracts.file.file_upload_path, validators=[arvestust.models.validators.file.validate_file_size, arvestust.models.validators.file.validate_file_type], verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='tag',
            name='content',
            field=models.CharField(max_length=64, unique=True, verbose_name='content'),
        ),
    ]



from django.utils import timezone
from django.views.generic import ListView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import path
from .urls import urlpatterns
from ..models import File


class FileListView(LoginRequiredMixin, ListView):
    model = File
    context_object_name = 'files'
    template_name = 'file_list.html'
    paginate_by = 20

    def get_queryset(self):
        return File.objects.filter(
            uploaded_by=self.request.user
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['now'] = timezone.now()
        return context


urlpatterns.append(
    path('files/', FileListView.as_view(), name='file-list')
)



from django.utils import timezone
from django.views.generic import DetailView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import path
from .urls import urlpatterns
from ..models import File
from ..forms import FileForm


class FileDetailView(LoginRequiredMixin, DetailView):
    model = File
    context_object_name = 'file'
    template_name = 'file_detail.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context


urlpatterns.append(
    path('files/<slug:slug>', FileDetailView.as_view(), name='file-detail')
)



# Generated by Django 2.2.14 on 2020-08-03 04:50

import arvestust.models.image
import arvestust.models.validators.file
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('arvestust', '0009_auto_20200730_1900'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('file', models.ImageField(storage='ekletik.settings.storage.PublicFileStorage', upload_to=arvestust.models.image.file_upload_path, validators=[arvestust.models.validators.file.validate_file_size], verbose_name='file')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('slug', models.SlugField(blank=True, editable=False, max_length=250, unique=True, verbose_name='slug')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='image', to='contenttypes.ContentType')),
            ],
            options={
                'db_table': 'arvestust_images',
                'ordering': ['-created_at'],
            },
        ),
        migrations.AddIndex(
            model_name='image',
            index=models.Index(fields=['created_at'], name='arvestust_i_created_1596b1_idx'),
        ),
    ]



from django import forms
from ..models.image import Image
from .arvestust_record import ArvestustRecordFormMixin


class ImageForm(ArvestustRecordFormMixin, forms.ModelForm):
    class Meta:
        model = Image
        exclude = ArvestustRecordFormMixin.Meta.exclude + ('uploaded_by',)



from django.test import TestCase
from ..image import Image


class ImageTestCase(TestCase):
    def setUp(self):
        """
        Create objects here...

        Example:
        Image.objects.create()
        """

        pass

    def test_create_image(self):
        """
        Run assertions here...

        Example:
        image = Image.objects.create()
        self.assertEqual(image.id, 1)
        """

        pass



# Generated by Django 2.2.14 on 2020-07-30 23:00

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
        ('arvestust', '0008_auto_20200730_1516'),
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('slug', models.SlugField(blank=True, editable=False, max_length=250, unique=True, verbose_name='slug')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('activity_type', models.CharField(choices=[('BOOKMARK', 'Bookmark'), ('FAVORITE', 'Favorite'), ('FOLLOW', 'Follow'), ('LIKE', 'Like'), ('REPORT', 'Report'), ('DOWN_VOTE', 'Down Vote'), ('UP_VOTE', 'Up Vote')], max_length=15)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='activity', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'db_table': 'arvestust_activities',
                'unique_together': {('object_id', 'user', 'content_type')},
            },
        ),
        migrations.AlterUniqueTogether(
            name='like',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='like',
            name='content_type',
        ),
        migrations.RemoveField(
            model_name='like',
            name='user',
        ),
        migrations.AlterUniqueTogether(
            name='save',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='save',
            name='content_type',
        ),
        migrations.RemoveField(
            model_name='save',
            name='user',
        ),
        migrations.RemoveField(
            model_name='comment',
            name='reports',
        ),
        migrations.DeleteModel(
            name='Follow',
        ),
        migrations.DeleteModel(
            name='Like',
        ),
        migrations.DeleteModel(
            name='Save',
        ),
    ]



# coding: utf-8
"""
    arvestust is a django app with support for file management.
"""
from __future__ import unicode_literals
from .requires import REQUIRED_APPS

__version__ = "0.7.0"
__license__ = 'BSD 3-Clause'
__copyright__ = 'Copyright 2020 Lehvitus ÖU'

VERSION = __version__

default_app_config = 'arvestust.apps.ArvestustConfig'



from rest_framework import serializers


class ArvestustRecordSerializerMixin(serializers.BaseSerializer):

    class Meta:
        exclude = (
            'uuid',
            'object_id',
            'content_type',
        )



from django import forms
from ..models.file import File
from .arvestust_record import ArvestustRecordFormMixin


class FileForm(ArvestustRecordFormMixin, forms.ModelForm):
    class Meta:
        model = File
        exclude = ArvestustRecordFormMixin.Meta.exclude + ('likes', 'uploaded_by')



import uuid
from django.db import models
from django.urls import reverse
from django.template.defaultfilters import slugify
from django.utils.translation import gettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey


class ArvestustRecord(models.Model):
    content_type = models.ForeignKey(
        ContentType,
        related_name='%(class)s',
        on_delete=models.CASCADE
    )
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    # Default fields. Used for record-keeping.
    uuid = models.UUIDField(default=uuid.uuid4, editable=False)
    slug = models.SlugField(_('slug'), max_length=250, unique=True, editable=False, blank=True)
    created_at = models.DateTimeField(_('created at'), auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(_('updated at'), auto_now=True, editable=False)

    class Meta:
        db_table = 'arvestust_records'
        indexes = [models.Index(fields=['created_at'])]
        ordering = ['-created_at']
        abstract = True

    def save(self, *args, **kwargs):
        # Generate a Medium-like URL slugs:
        self.slug = slugify(f'{str(self.uuid)}')
        super().save(*args, **kwargs)

    def __str__(self):
        return f'{self.slug}'

    def get_absolute_url(self):
        return reverse('%(class)-detail', kwargs={'slug': self.slug})



from rest_framework import serializers
from ..models import Comment


class CommentSerializer(serializers.ModelSerializer):

    class Meta:
        model = Comment
        exclude = (
            'uuid',
            'object_id',
            'content_type',
            'reports',
        )



# Generated by Django 2.2.14 on 2020-07-30 19:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('arvestust', '0006_auto_20200730_1513'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='tag',
            unique_together=set(),
        ),
    ]



from django.views.generic import DetailView
from django.urls import path
from .urls import urlpatterns
from ..models import Image


class ImageDetailView(DetailView):
    model = Image
    context_object_name = 'image'
    template_name = 'image_detail.html'
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context


urlpatterns.append(
    path('images/<slug:slug>', ImageDetailView.as_view(), name='image-detail')
)



from rest_framework import viewsets
from rest_framework import permissions
from rest_framework.decorators import action
from rest_framework.response import Response


class ArvestustFileMixin(viewsets.ModelViewSet):
    permission_classes = [permissions.IsAuthenticated]

    def get_queryset(self):
        return self.queryset.get_for_user(
            user=self.request.user
        )

    @action(detail=False, methods=['get'])
    def me(self, request):
        objects = self.queryset.objects.filter(author=request.user)

        page = self.paginate_queryset(objects)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(objects, many=True)
        return Response(serializer.data)



from django.views.generic import ListView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import path
from .urls import urlpatterns
from ..models import Tag


class TagListView(LoginRequiredMixin, ListView):
    model = Tag
    context_object_name = 'tags'
    template_name = 'tag_list.html'
    paginate_by = 100

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context


urlpatterns.append(
    path('tags/', TagListView.as_view(), name='tag-list')
)



import os
import inflection
import magic
from django.db import models
from django.urls import reverse
from django.contrib.auth import get_user_model
from django.template.defaultfilters import slugify
from django.utils.translation import gettext_lazy as _
from django.conf import settings
from .record import ArvestustRecord
from ..managers.arvestust_file import ArvestustFileManager
from ..validators.file import validate_file_size, validate_file_type, validate_storage_quota


def file_upload_path(instance, filename):
    name, extension = os.path.splitext(filename)
    file = f'{str(instance.uuid)}{extension}'
    folder = inflection.pluralize(str(instance.content_type))
    return f'{folder}/{instance.content_object.uuid}/{file}'


def check_in_memory_mime(in_memory_file):
    mime = magic.from_buffer(in_memory_file.read(), mime=True)
    return mime


class ArvestustFile(ArvestustRecord):
    name = models.CharField(max_length=50, blank=True)
    file = models.FileField(
        upload_to=file_upload_path,
        verbose_name=_('file'),
        storage=getattr(settings, 'PUBLIC_FILE_STORAGE'),
    )
    mime = models.CharField(max_length=100, blank=True, editable=False)
    is_private = models.BooleanField(default=False)

    # TODO: Handle ArvestustRecord deletion
    # Determine the content_type associated with the upload file then opt for the best course of action.
    # The goal is to be compliant with GDPR and other regulations while also ensuring the availability of
    # any materials that are for public consumption on the platform.
    uploaded_by = models.ForeignKey(
        get_user_model(),
        related_name='%(class)s',
        on_delete=models.DO_NOTHING,
        editable=False,
        blank=True,
        null=True
    )

    class Meta:
        db_table = 'arvestust_records'
        abstract = True

    objects = ArvestustFileManager()

    def clean(self, *args, **kwargs):
        self.mime = check_in_memory_mime(self.file)
        super(ArvestustFile, self).clean(*args, **kwargs)

    def save(self, *args, **kwargs):
        # Generate a Medium-like URL slugs:
        self.slug = slugify(f'{self.name}-{str(self.uuid)[-12:]}')

        if self.is_private:
            # self.file.storage = settings.PRIVATE_FILE_STORAGE
            self.file.storage = getattr(settings, 'PRIVATE_FILE_STORAGE')

        super().save(*args, **kwargs)

    def delete(self, using=None, keep_parents=False):
        self.file.storage.delete(self.file.name)
        super().delete()

    def __str__(self):
        return self.name if self.name else f'{self.slug}'

    def get_absolute_url(self):
        return reverse('%(class)-detail', kwargs={'slug': self.slug})



from django.test import TestCase
from ..models.comment import Comment


class CommentTestCase(TestCase):
    def setUp(self):
        """
        Create objects here...

        Example:
        Comment.objects.create()
        """
        pass

    def test_create_comment(self):
        """
        Run assertions here...

        Example:
        comment = Comment.objects.create()
        self.assertEqual(comment.id, 1)
        """
        pass



from django.views.generic import DetailView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import path
from .urls import urlpatterns
from ..models import Tag


class TagDetailView(LoginRequiredMixin, DetailView):
    model = Tag
    context_object_name = 'tag'
    template_name = 'tag_detail.html'
    paginate_by = 20

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context


urlpatterns.append(
    path('tags/<slug:slug>', TagDetailView.as_view(), name='tag-detail')
)



from rest_framework import routers

"""
To make this app's urls available to the entire project,
include the app urls in the urlpatterns for your project:

In your project's urls.py file (the one close to settings.py) do so:

from django.urls import include, path
urlpatterns = [
   # Your other url patterns...
   path('files/', include('arvestust.api', namespace='arvestust_api'))
]

"""

router = routers.SimpleRouter(trailing_slash=False)



# arvestust:views
from .file_detail import FileDetailView
from .file_list import FileListView
from .comment_detail import CommentDetailView
from .comment_list import CommentListView
from .tag_detail import TagDetailView
from .tag_list import TagListView
from .image_detail import ImageDetailView
from .image_list import ImageListView



from rest_framework import viewsets
from rest_framework import permissions
from arvestust.viewsets.router import router
from arvestust.models import File
from arvestust.serializers import FileSerializer
from arvestust.viewsets.mixins import ArvestustFileMixin


class FileViewSet(ArvestustFileMixin, viewsets.ModelViewSet):
    queryset = File.objects.all()
    serializer_class = FileSerializer
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]


router.register('files', FileViewSet)



from django.db import models
from django.db.models import Q


class UserRecordManager(models.Manager):

    def get_queryset(self):
        return super().get_queryset().prefetch_related()

    def get_for_user(self, user):
        return self.get_queryset().filter(user=user)

    def get_matches(self, pk, user, obj_id):
        return self.get_queryset().exclude(id=pk).filter(
            Q(user=user) &
            Q(object_id=obj_id)
        ).distinct()



from rest_framework import viewsets
from rest_framework import permissions
from arvestust.viewsets.router import router
from arvestust.models import Comment
from arvestust.serializers import CommentSerializer
from arvestust.viewsets.mixins import ArvestustRecordMixin


class CommentViewSet(ArvestustRecordMixin, viewsets.ModelViewSet):
    queryset = Comment.objects.all()
    serializer_class = CommentSerializer
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    def get_queryset(self):
        if self.request.user.is_authenticated:
            return Comment.objects.get_for_user(
                user=self.request.user
            )
        return self.queryset


router.register('comments', CommentViewSet)



import os
import uuid
import inflection
from django.db import models
from django.urls import reverse
from django.template.defaultfilters import slugify
from django.utils.translation import gettext_lazy as _
from django.conf import settings
from .abstracts import ArvestustRecord, ArvestustFile
from .validators.file import validate_file_size, validate_storage_quota


def file_upload_path(instance, filename):
    name, extension = os.path.splitext(filename)
    file = f'{str(instance.uuid)}{extension}'
    folder = inflection.pluralize(str(instance.content_type))
    return f'{folder}/{instance.content_object.uuid}/{file}'


class Image(ArvestustFile):
    file = models.ImageField(
        upload_to=file_upload_path,
        verbose_name=_('file'),
        storage=getattr(settings, 'PUBLIC_FILE_STORAGE'),
    )

    class Meta:
        db_table = 'arvestust_images'
        indexes = [models.Index(fields=['created_at'])]
        ordering = ['-created_at']

    def get_absolute_url(self):
        return reverse('image-detail', kwargs={'slug': self.slug})



# arvestust:audit
from auditlog.registry import auditlog
from ..models import Activity
from ..models import Comment
from ..models import File
from ..models import Image
from ..models import Tag

auditlog.register(Activity)
auditlog.register(Comment)
auditlog.register(File)
auditlog.register(Image)
auditlog.register(Tag)



from rest_framework import status
from rest_framework.test import APIClient, APITestCase
from django.contrib.auth import get_user_model
from ..tag import Tag


class TagTestCase(APITestCase):

    # The client used to connect to the API
    client = APIClient()

    def setUp(self):
        """
        Prepare database and client.
        """

        # API endpoint
        self.namespace = '/v1/tags'

    @classmethod
    def setUpTestData(cls):
        # Create users
        cls.alice = get_user_model().objects.create(username="alice", email="alice@example.org")
        cls.bob = get_user_model().objects.create(username="bob", email="bob@example.org")

        # Create tags
        # cls.tag1 = Tag.objects.create(...)
        # cls.tag2 = Tag.objects.create(...)
        # cls.tag3 = Tag.objects.create(...)

    #################################################################
    # Require authentication
    def test_must_authenticate_to_read_tags(self):
        res = self.client.get(self.namespace)
        self.assertEqual(res.status_code, status.HTTP_403_FORBIDDEN)

    def test_must_authenticate_to_create_tags(self):
        res = self.client.post(self.namespace)
        self.assertEqual(res.status_code, status.HTTP_403_FORBIDDEN)

    #################################################################
    # Allowed requests
    def test_create_tag(self):
        self.client.force_authenticate(user=self.alice)

        res = self.client.post(self.namespace, data={})
        self.assertEqual(res.status_code, status.HTTP_201_CREATED)

    def test_list_tag(self):
        self.client.force_authenticate(user=self.alice)

        res = self.client.get(self.namespace)
        self.assertEqual(res.status_code, status.HTTP_200_OK)

    def test_retrieve_tag(self):
        self.client.force_authenticate(user=self.alice)

        url = self.namespace + '/1'
        res = self.client.get(url)
        self.assertEqual(res.status_code, status.HTTP_200_OK)

    def test_update_tag(self):
        self.client.force_authenticate(user=self.alice)

        url = self.namespace + '/1'
        res = self.client.patch(url, data={})
        self.assertEqual(res.status_code, status.HTTP_202_ACCEPTED)

    def test_delete_tag(self):
        self.client.force_authenticate(user=self.alice)

        url = self.namespace + '/1'
        res = self.client.delete(url)
        self.assertEqual(res.status_code, status.HTTP_204_NO_CONTENT)



# Generated by Django 2.2.13 on 2020-08-03 20:19

import arvestust.models.abstracts.file
import arvestust.models.image
import arvestust.models.validators.file
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('arvestust', '0010_auto_20200803_0050'),
    ]

    operations = [
        migrations.AddField(
            model_name='image',
            name='is_private',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='image',
            name='mime',
            field=models.CharField(blank=True, editable=False, max_length=100),
        ),
        migrations.AddField(
            model_name='image',
            name='name',
            field=models.CharField(blank=True, max_length=50),
        ),
        migrations.AddField(
            model_name='image',
            name='uploaded_by',
            field=models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='image', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='file',
            name='file',
            field=models.FileField(storage='packager.storage.PublicFileStorage', upload_to=arvestust.models.abstracts.file.file_upload_path, validators=[arvestust.models.validators.file.validate_file_size, arvestust.models.validators.file.validate_file_type], verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='image',
            name='file',
            field=models.ImageField(storage='packager.storage.PublicFileStorage', upload_to=arvestust.models.image.file_upload_path, validators=[arvestust.models.validators.file.validate_file_size], verbose_name='file'),
        ),
    ]



from django.test import TestCase
from ..models.like import Like


class LikeTestCase(TestCase):
    def setUp(self):
        """
        Create objects here...

        Example:
        Like.objects.create()
        """

        pass

    def test_create_like(self):
        """
        Run assertions here...

        Example:
        like = Like.objects.create()
        self.assertEqual(like.id, 1)
        """

        pass



from django.test import TestCase
from ..models.tag import Tag


class TagTestCase(TestCase):
    def setUp(self):
        """
        Create objects here...

        Example:
        Tag.objects.create()
        """

        pass

    def test_create_tag(self):
        """
        Run assertions here...

        Example:
        tag = Tag.objects.create()
        self.assertEqual(tag.id, 1)
        """

        pass



# arvestust:models:validators
import magic
from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError

__MAX_UPLOAD_SIZE__ = 10 << 20  # 10 Megabytes

__DEFAULT_ORGANIZATION_QUOTA__ = 10 << 30  # 10 Gigabytes

__DEFAULT_USER_QUOTA__ = 100 << 20  # 100 Megabytes

VALID_AUDIO_TYPES = [
    'audio/3gpp',
    'audio/3gpp2',
    'audio/aac',
    'audio/midi',
    'audio/mpeg',
    'audio/ogg',
    'audio/opus',
    'audio/wav',
    'audio/webm',
    'audio/x-midi',
]

VALID_IMAGE_TYPES = [
    'image/bmp',
    'image/gif',
    'image/ief',
    'image/jp2',
    'image/jpeg',
    'image/png',
    'image/svg+xml',
    'image/vnd.microsoft.icon',
    'image/vnd.wap.wbmp',
    'image/tiff',
    'image/x-icon',
]

VALID_VIDEO_TYPES = [
    'video/3gpp',
    'video/3gpp2',
    'video/mp4',
    'video/mpeg',
    'video/ogg',
    'video/webm',
    'video/x-msvideo',
]

VALID_TEXT_TYPES = [
    'text/calendar',
    'text/css',
    'text/csv',
    'text/html',
    'text/javascript',
    'text/plain',
    'text/richtext',
    'text/rtf'
]

VALID_APPLICATION_TYPES = [
    'application/epub+zip',
    'application/json',
    'application/msword',
    'application/pdf',
    'application/php',
    'application/vnd.apple.keynote',
    'application/vnd.apple.pages',
    'application/vnd.apple.numbers',
    'application/vnd.ms-excel',
    'application/vnd.ms-powerpoint',
    'application/vnd.oasis.opendocument.presentation',
    'application/vnd.oasis.opendocument.spreadsheet',
    'application/vnd.oasis.opendocument.text',
    'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    'application/x-tar',
    'application/xhtml+xml',
    'application/xml',
    'application/xml-dtd',
    'application/xslt+xml',
    'application/zip'
]

VALID_FILE_TYPES = {
    'audio': VALID_AUDIO_TYPES,
    'image': VALID_IMAGE_TYPES,
    'video': VALID_VIDEO_TYPES,
    'text': VALID_TEXT_TYPES,
    'application': VALID_APPLICATION_TYPES
}


def validate_file_type(file):
    v = sorted(v for value in VALID_FILE_TYPES.values() for v in value)
    mime = magic.from_buffer(file.read(), mime=True)
    if mime in v:
        return mime
    raise ValidationError(_('This file type is not supported'))


def validate_file_size(file):
    if file.size > __MAX_UPLOAD_SIZE__:
        raise ValidationError(_(f'Maximum file size is {__MAX_UPLOAD_SIZE__}'))
    return file


def validate_storage_quota(file):
    # TODO: before uploading, determine if user or organization has if within upload limits
    pass



from rest_framework import viewsets
from rest_framework import permissions
from .router import router
from ..models import Tag
from ..serializers import TagSerializer
from .mixins import ArvestustRecordMixin


class TagViewSet(ArvestustRecordMixin, viewsets.ModelViewSet):
    queryset = Tag.objects.all()
    serializer_class = TagSerializer
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]


router.register('tags', TagViewSet)



from django.contrib.contenttypes.fields import GenericRelation
from django.urls import reverse
from .abstracts import ArvestustFile
from .comment import Comment
from .activity import Activity


class File(ArvestustFile):
    comments = GenericRelation(Comment, related_query_name='files')

    likes = GenericRelation(Activity, related_query_name='comments')
    votes = GenericRelation(Activity, related_query_name='comments')
    saves = GenericRelation(Activity, related_query_name='comments')
    reports = GenericRelation(Activity, related_query_name='comments')

    class Meta:
        db_table = 'arvestust_files'

    def get_absolute_url(self):
        return reverse('arvestust:file-detail', kwargs={'slug': self.slug})



from django import forms
from ..models.abstracts import ArvestustRecord


class ArvestustRecordFormMixin(forms.BaseModelForm):
    class Meta:
        exclude = (
            'content_type',
            'object_id',
            'content_object',
        )



from django.db import models
from django.db.models import Q


class FileManager(models.Manager):

    def get_queryset(self):
        return super().get_queryset().prefetch_related()

    def get_for_user(self, user):
        return self.get_queryset().filter(user=user)



# Generated by Django 2.2.14 on 2020-07-30 19:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('arvestust', '0007_auto_20200730_1513'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tag',
            name='content',
            field=models.CharField(max_length=64, verbose_name='content'),
        ),
        migrations.AlterUniqueTogether(
            name='tag',
            unique_together={('object_id', 'content_type', 'content')},
        ),
    ]



# arvestust:models:tests
from .file import FileTestCase
from .like import LikeTestCase
from .tag import TagTestCase
from .follow import FollowTestCase
from .save import SaveTestCase
from .image import ImageTestCase



from django.utils import timezone
from django.views.generic import DetailView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import path
from ..models import Comment
from .urls import urlpatterns


class CommentDetailView(LoginRequiredMixin, DetailView):
    model = Comment
    context_object_name = 'comment'
    template_name = 'comment_detail.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context


urlpatterns.append(
    path('comments/<slug:slug>', CommentDetailView.as_view(), name='comment-detail')
)



# arvestust:admin
from django.contrib import admin
from ..models import Comment


@admin.register(Comment)
class CommentAdmin(admin.ModelAdmin):
    readonly_fields = ['user']
    list_display = [
        'slug',
        'user',
        'created_at',
        'updated_at',
    ]

    # Ensure current user is assigned as author of post instance.
    def save_model(self, request, obj, form, change):
        if not obj.user_id:
            obj.user = request.user
        obj.save()



from django.views.generic import ListView
from django.urls import path
from .urls import urlpatterns
from ..models import Image


class ImageListView(ListView):
    model = Image
    context_object_name = 'images'
    template_name = 'image_list.html'
    paginate_by = 20
    
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context


urlpatterns.append(
    path('images/', ImageListView.as_view(), name='image-list')
)



# arvestust:apps
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ArvestustConfig(AppConfig):
    name = 'arvestust'
    verbose_name = _('Arvestust Files and Record-Keeping')

    def ready(self):
        from .models import signals
        from . import audit



from django.contrib import admin
from ..models import Tag


@admin.register(Tag)
class TagAdmin(admin.ModelAdmin):
    list_display = [
        'slug',
        'created_at',
        'updated_at',
    ]
    search_fields = ['content']



from django.test import TestCase
from ..models.follow import Follow


class FollowTestCase(TestCase):
    def setUp(self):
        """
        Create objects here...

        Example:
        Follow.objects.create()
        """

        pass

    def test_create_follow(self):
        """
        Run assertions here...

        Example:
        follow = Follow.objects.create()
        self.assertEqual(follow.id, 1)
        """

        pass



from django import forms
from ..models.comment import Comment
from .arvestust_record import ArvestustRecordFormMixin


class CommentForm(ArvestustRecordFormMixin, forms.ModelForm):
    class Meta(ArvestustRecordFormMixin.Meta):
        model = Comment
        exclude = ArvestustRecordFormMixin.Meta.exclude + ('reports', 'likes', 'user')



from rest_framework import viewsets
from rest_framework import permissions
from .router import router
from ..models import Image
from ..serializers import ImageSerializer


class ImageViewSet(viewsets.ModelViewSet):
    queryset = Image.objects.all()
    serializer_class = ImageSerializer
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]


router.register('images', ImageViewSet)



# arvestust:admin
from django.contrib import admin
from ..models import File


@admin.register(File)
class FileAdmin(admin.ModelAdmin):
    readonly_fields = ['uploaded_by']
    list_display = [
        'slug',
        'created_at',
        'updated_at',
        'uploaded_by',
        'content_type',
        'object_id',
        'file',
    ]
    # autocomplete_fields = ['content_type']

    # Ensure current user is assigned as author of post instance.
    def save_model(self, request, obj, form, change):
        if not obj.uploaded_by_id:
            obj.uploaded_by = request.user
        obj.save()



from django.db import models
from django.db.models import Q


class CommentManager(models.Manager):

    def get_queryset(self):
        return super().get_queryset().prefetch_related()

    def get_for_user(self, user):
        return self.get_queryset().filter(
            (
                # TODO: include comments liked by user
                # Q(likes_author__in=user) |
                ~Q(reports__in=[user])
            )
        )



from django.db import models
from django.contrib.contenttypes.fields import GenericRelation
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model
from .managers import CommentManager
from .abstracts import ArvestustRecord
from .activity import Activity


class Comment(ArvestustRecord):
    user = models.ForeignKey(
        get_user_model(),
        related_name='comments',
        on_delete=models.DO_NOTHING,
        verbose_name=_('author')
    )
    content = models.TextField(blank=True, verbose_name=_('content'))

    likes = GenericRelation(Activity, related_query_name='comments')
    votes = GenericRelation(Activity, related_query_name='comments')
    saves = GenericRelation(Activity, related_query_name='comments')
    reports = GenericRelation(Activity, related_query_name='comments')

    def is_inappropriate(self):
        return self.reports.count() >= 5

    def has_likes(self):
        return self.likes.count() > 0

    class Meta:
        db_table = 'arvestust_comments'
        indexes = [models.Index(fields=['user', 'created_at'])]
        ordering = ['-created_at']

    objects = CommentManager()

    def get_absolute_url(self):
        return reverse('arvestust:comment-detail', kwargs={'slug': self.slug})



import os
import re
from setuptools import find_packages, setup


def read(f):
    return open(f, 'r', encoding='utf-8').read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('arvestust')

setup(
    version=version,
    name='django-arvestust',
    url='https://www.github.com/lehvitus/arvestust',
    author='Leo Neto',
    author_email='projects@lehvitus.com',
    description='A Django app for file management and record-keeping',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
)



from django.db import models
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError
from .managers import ActivityManager
from .abstracts import ArvestustRecord


class Activity(ArvestustRecord):

    FAVORITE = 'FAVORITE'
    FOLLOW = 'FOLLOW'
    BOOKMARK = 'BOOKMARK'
    LIKE = 'LIKE'
    REPORT = 'REPORT'
    VOTE_DOWN = 'DOWN_VOTE'
    VOTE_UP = 'UP_VOTE'

    ACTIVITY_TYPES = (
        (BOOKMARK, 'Bookmark'),
        (FAVORITE, 'Favorite'),
        (FOLLOW, 'Follow'),
        (LIKE, 'Like'),
        (REPORT, 'Report'),
        (VOTE_DOWN, 'Down Vote'),
        (VOTE_UP, 'Up Vote'),
    )

    user = models.ForeignKey(
        get_user_model(),
        on_delete=models.CASCADE,
        verbose_name=_('user')
    )
    activity_type = models.CharField(
        max_length=15,
        choices=ACTIVITY_TYPES
    )

    class Meta:
        db_table = 'arvestust_activities'
        unique_together = ('object_id', 'user', 'content_type')

    objects = ActivityManager()

    def clean(self):
        super().clean()
        qs = Activity.objects.get_matches(
            pk=self.pk,
            user=self.user,
            obj_id=self.object_id,
        )
        if qs.count() > 0:
            raise ValidationError(_('User has already added this content.'))

    def get_absolute_url(self):
        return reverse('arvestust:activity-detail', kwargs={'slug': self.slug})



# Generated by Django 2.2.13 on 2020-07-23 22:14

import arvestust.models.abstracts.file
import arvestust.models.validators.file
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('slug', models.SlugField(blank=True, editable=False, max_length=250, unique=True, verbose_name='slug')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('content', models.CharField(max_length=64, verbose_name='content')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tag', to='contenttypes.ContentType')),
            ],
            options={
                'db_table': 'arvestust_tags',
            },
        ),
        migrations.CreateModel(
            name='Save',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('slug', models.SlugField(blank=True, editable=False, max_length=250, unique=True, verbose_name='slug')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('notes', models.CharField(blank=True, max_length=255, null=True, verbose_name='notes')),
                ('permalink', models.URLField(blank=True, null=True, verbose_name='permalink')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='save', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='saves', to=settings.AUTH_USER_MODEL, verbose_name='author')),
            ],
            options={
                'db_table': 'arvestust_saves',
            },
        ),
        migrations.CreateModel(
            name='Like',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('slug', models.SlugField(blank=True, editable=False, max_length=250, unique=True, verbose_name='slug')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='like', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likes', to=settings.AUTH_USER_MODEL, verbose_name='author')),
            ],
            options={
                'db_table': 'arvestust_likes',
            },
        ),
        migrations.CreateModel(
            name='Follow',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('slug', models.SlugField(blank=True, editable=False, max_length=250, unique=True, verbose_name='slug')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='follow', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='follows', to=settings.AUTH_USER_MODEL, verbose_name='author')),
            ],
            options={
                'db_table': 'arvestust_follows',
            },
        ),
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('slug', models.SlugField(blank=True, editable=False, max_length=250, unique=True, verbose_name='slug')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('name', models.CharField(blank=True, max_length=50)),
                ('file', models.FileField(upload_to=arvestust.models.abstracts.file.file_upload_path, validators=[arvestust.models.validators.file.validate_file_size, arvestust.models.validators.file.validate_file_type], verbose_name='file')),
                ('mime', models.CharField(blank=True, editable=False, max_length=100)),
                ('is_private', models.BooleanField(default=False)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='file', to='contenttypes.ContentType')),
                ('uploaded_by', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='file', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'arvestust_files',
            },
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('slug', models.SlugField(blank=True, editable=False, max_length=250, unique=True, verbose_name='slug')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('content', models.TextField(blank=True, verbose_name='content')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comment', to='contenttypes.ContentType')),
                ('reports', models.ManyToManyField(blank=True, related_name='reported_comments', to=settings.AUTH_USER_MODEL, verbose_name='reports')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='comments', to=settings.AUTH_USER_MODEL, verbose_name='author')),
            ],
            options={
                'db_table': 'arvestust_comments',
                'ordering': ['-created_at'],
            },
        ),
        migrations.AddIndex(
            model_name='follow',
            index=models.Index(fields=['user', 'created_at'], name='arvestust_f_user_id_3052ae_idx'),
        ),
        migrations.AddIndex(
            model_name='comment',
            index=models.Index(fields=['user', 'created_at'], name='arvestust_c_user_id_429c1c_idx'),
        ),
    ]



from django.utils import timezone
from django.views.generic import ListView, CreateView
from django.urls import path
from django.contrib.auth.mixins import LoginRequiredMixin
from .urls import urlpatterns
from ..models import Comment
from ..forms import CommentForm


class CommentListView(LoginRequiredMixin, ListView, CreateView):
    model = Comment
    context_object_name = 'comments'
    template_name = 'comment_list.html'
    form_class = CommentForm
    paginate_by = 50

    def form_valid(self, form):
        form.instance.author = self.request.user
        return super().form_valid(form)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context


urlpatterns.append(
    path('comments/', CommentListView.as_view(), name='comment-list')
)



from rest_framework import serializers
from ..models import Tag


class TagSerializer(serializers.ModelSerializer):

    class Meta:
        model = Tag
        exclude = (
            'uuid',
            'object_id',
            'content_type',
        )



# Generated by Django 2.2.13 on 2020-07-29 16:53

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('arvestust', '0002_auto_20200729_1639'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='follow',
            unique_together={('object_id', 'user')},
        ),
        migrations.AlterUniqueTogether(
            name='like',
            unique_together={('object_id', 'user')},
        ),
        migrations.AlterUniqueTogether(
            name='save',
            unique_together={('object_id', 'user')},
        ),
        migrations.AlterUniqueTogether(
            name='tag',
            unique_together={('object_id', 'content')},
        ),
    ]



from django import forms
from ..models.tag import Tag
from .arvestust_record import ArvestustRecordFormMixin


class TagForm(ArvestustRecordFormMixin, forms.ModelForm):
    class Meta:
        model = Tag
        exclude = ArvestustRecordFormMixin.Meta.exclude + ('user',)
